//
//  EOEditingContextInspector.java
//  EOInspector
//

import java.util.*;
import java.text.*; 
import com.webobjects.foundation.*;
import com.webobjects.eoaccess.*;
import com.webobjects.eocontrol.*;

public class EOEditingContextInspector extends Inspector {
    String entityNameToFetch;
    EOQualifier qualifierToFetch;
    int limitToFetch;
    NSArray fetchedObjects;
    Logger delegateLogger;

    public EOEditingContextInspector() {
        super();
        entityNameToFetch = "Movie";
        limitToFetch = 1;
        delegateLogger = DelegateLogger.loggerWithDelegateLogger();
    }

	public String bindingBaseName() {
		return "ec";
	}
	
    private EOEditingContext ec() {
        return (EOEditingContext)object();
    }

    public NSArray allKeys() {
        return new NSArray(new Object[] {
            "defaultFetchTimestampLag", "fetchTimestamp", "fetchedObjects",
            "insertedObjects", "updatedObjects", "deletedObjects", "rootObjectStore",
            "entityNameToFetch", "qualifierToFetch", "limitToFetch",
            "delegateLogger"});
    }

    public NSArray allActions() {
        return new NSArray(new Object[] {
            "setFetchTimestampToNow", "fetchObjects", "createAndInsertInstance",
            "saveChanges", "revert", "invalidateAllObjects", "refaultAllObjects",
            "refreshAllObjects"});
    }

    public long defaultFetchTimestampLag() {
        return EOEditingContext.defaultFetchTimestampLag();
    }

    public void setDefaultFetchTimestampLag(String value) {
        EOEditingContext.setDefaultFetchTimestampLag(Long.parseLong(value));
    }

    public NSTimestamp fetchTimestamp() {
        return new NSTimestamp(ec().fetchTimestamp());
    }

    public void setFetchTimestamp(String timestamp) {
        try {
            ec().setFetchTimestamp(DateFormat.getInstance().parse(timestamp).getTime());
        } catch (ParseException e) {
            System.out.println("ParseException: " + timestamp);
        }
    }

    public String descriptionForObjects(NSArray objects) {
        String desc = "(";
        EOEnterpriseObject eo;
        EOGlobalID gid;

        Enumeration e = objects.objectEnumerator();
        while (e.hasMoreElements()) {
            eo = (EOEnterpriseObject)e.nextElement();
            gid = eo.editingContext().globalIDForObject(eo);
            desc = gid.toString();
            if (objects.lastObject().equals(eo) == false)
                desc = desc + ", ";
        }
        desc = desc + ")";
        return desc;
    }

    public void setFetchTimestampToNow() {
        ec().setFetchTimestamp(new NSTimestamp().getTime());
    }

    public void createAndInsertInstance() {
        if ((entityNameToFetch != null) &&
            (entityNameToFetch.equals("") == false)) {
            EOUtilities.createAndInsertInstance(ec(), entityNameToFetch);
        }
    }

    public void fetchObjects() {
        EOFetchSpecification fs = 
            new EOFetchSpecification(entityNameToFetch, qualifierToFetch, null);
        fs.setFetchLimit(limitToFetch);
        fetchedObjects = ec().objectsWithFetchSpecification(fs);

        Enumeration e = fetchedObjects.objectEnumerator();
        while (e.hasMoreElements()) {
            EOObserverCenter.addObserver(DelegateLogger.defaultLogger(),
                                         e.nextElement());
        }
    }

    public Object inspectorWillTakeValueForKey(Object value, String key) {
        if (key.equals("limitToFetch")) {
            return new Integer(value.toString());
        } else if (key.equals("qualifierToFetch")) {
            return EOQualifier.qualifierWithQualifierFormat(value.toString(), new NSArray());
        }

        return null;
    }

    public void updateLoggers() {
        delegateLogger.setTarget(object());
        delegateLogger.update();
    }

}
