//
// InspectPage.java: Class file for WO Component 'InspectPage'
// Project EOInspector
//
// Created by john on Wed Jul 21 2004
//

import com.webobjects.foundation.*;
import com.webobjects.appserver.*;
import com.webobjects.eocontrol.*;
import com.webobjects.eoaccess.*;
import groovy.lang.*;

public class InspectPage extends WOComponent {
    Object object;
    String keyName;
    String actionName;
    String description;
    Inspector inspector;
    protected String eventLog;
    protected String valueToSend;
    public String selectedKey;
    public String script;

    public InspectPage(WOContext context) {
        super(context);
    }

    public Inspector inspector() {
        return inspector;
    }

    public void setInspector(Inspector newInspector) {
        inspector = newInspector;
    }

    public Object object()
    {
        return object;
    }

    public void setObject(Object newObject)
    {
        object = newObject;
        inspector = (Inspector)inspectorForObject(newObject);
        inspector.setObject(object);
    }

    public static Inspector inspectorForObject(Object object) {
        Class inspectorClass, objectClass;
        Inspector newInspector = null;
        String baseClassName, className;

        baseClassName = null;
        objectClass = object.getClass();
        while (newInspector == null) {
            baseClassName = Inspector.baseClassName(objectClass);
            className = baseClassName + "Inspector";
            try {
                inspectorClass = Class.forName(className);
                newInspector = (Inspector)inspectorClass.newInstance();
            } catch (ClassNotFoundException e) {
                newInspector = null;
            } catch (InstantiationException e) {
                System.out.println("InstantiationException: " + className);
                newInspector = null;
            } catch (IllegalAccessException e) {
                System.out.println("IllegalAccessException: " + className);
                newInspector = null;
            } catch (ClassCastException e) {
                System.out.println("ClassCastException: " + className);
                newInspector = null;
            }
            objectClass = objectClass.getSuperclass();
        }

        if (newInspector == null) newInspector = new ObjectInspector();
        newInspector.setObject(object);
        newInspector.inspectorDidSetObject();
        newInspector.updateLoggers();
        return newInspector;
    }

    /** @TypeInfo java.lang.String */
    public NSArray allKeys() {
        return inspector.allKeys();
    }

    public String keyName()
    {
        return keyName;
    }

    public void setKeyName(String newKeyName)
    {
        keyName = newKeyName;
    }

    /** @TypeInfo java.lang.String */
    public NSArray allActions() {
        return inspector.allActions();
    }

    public WOComponent invokeSelectedAction() throws Exception
    {
        Object value = null;

        eventLog = null;
        String transactionName = inspector.name() + "." + actionName;

        DelegateLogger.defaultLogger().beginTransaction(transactionName);
        try {
            value = inspector.valueForKey(actionName);
        } catch (Exception e) {
            DelegateLogger.defaultLogger().endTransaction();
            throw e;
        }

        if (value != null) description = value.toString();
        DelegateLogger.defaultLogger().endTransaction();
        inspector.inspectorDidInvokeAction();
        inspector.updateLoggers();
        return null;
    }

    public WOComponent displayValue()
    {
        Object value = inspector.valueForKey(keyName);
        if (value != null)
            description = value.toString();
        else
            description = null;
        eventLog = null;
        return null;
    }

    public WOComponent editValue()
    {
        return null;
    }

    public String description()
    {
        return description;
    }

    public WOComponent displayInspector()
    {
        Object value = inspector.valueForKey(keyName);
        if (value != null)
            ObjectBrowser.defaultBrowser().registerObject(value);
        return null;
    }

    public String eventLog()
    {
        return eventLog;
    }

    public String valueToSend()
    {
        return valueToSend;
    }

    public void setValueToSend(String newValueToSend)
    {
        valueToSend = newValueToSend;
    }

    public WOComponent sendMessage()
    {
        eventLog = null;
        try {
            inspector.takeValueForKey(valueToSend, selectedKey);
        } catch (Exception e) {
            eventLog = e.toString();
        }

        return null;
    }

    public boolean hasEventLog()
    {
        if (eventLog != null)
            return true;
        else
            return false;
    }
	
    public WOComponent evaluate()
    {
		try {
			GroovyShell sh = ObjectBrowser.defaultBrowser().workspace().shell();
			Closure cl = (Closure)sh.evaluate("cl = {" + script + "}");
			cl.call(inspector.object());
		} catch (Exception e) {
			eventLog = e.toString();
		}
        return null;
    }

}
