//
//  Inspection.java
//  EOInspector
//

import com.webobjects.foundation.*;
import com.webobjects.eocontrol.*;

public abstract class Inspector {
    Object object;
    String name;
	int count;

    public static boolean canAccessFieldsDirectly() {
        return true;
    }

    public static String baseClassName(Object object) {
        String[] splitted = object.getClass().getName().split("\\.");
        return splitted[splitted.length-1];
    }

    public static String baseClassName(Class klass) {
        String[] splitted = klass.getName().split("\\.");
        return splitted[splitted.length-1];
    }

    public Object object()
    {
        return object;
    }

    public void setObject(Object newObject)
    {
        object = newObject;
    }

    public String name() {
        if (name == null)
            return baseClassName(object);
        else
            return name;
    }

    public void setName(String newName) {
        name = newName;
    }

	public void setCount(int newCount) {
		count = newCount;
	}
	
	// binding for groovy
	public abstract String bindingBaseName();
	public String bindingName() {
		return bindingBaseName() + count;
	}
	
    public Object valueForKey(String key) {
        try {
            return NSKeyValueCoding.DefaultImplementation.valueForKey(this, key);
        } catch (NSKeyValueCoding.UnknownKeyException e) {
            return NSKeyValueCoding.DefaultImplementation.valueForKey(object(), key);
        }
    }

    public void takeValueForKey(Object value, String key) {
        Object evaluated;
        try {
            evaluated =    this.inspectorWillTakeValueForKey(value, key);
            if (evaluated != null)
                value = evaluated;
        } catch (NSKeyValueCoding.UnknownKeyException e) {
        }

        try {
            NSKeyValueCoding.DefaultImplementation.takeValueForKey(this, value, key);
        } catch (NSKeyValueCoding.UnknownKeyException e) {
            NSKeyValueCoding.DefaultImplementation.takeValueForKey(object(), value, key);
        }
    }

    public abstract NSArray allKeys();
    public abstract NSArray allActions();

    public void registerObject(Object object) {
        ObjectBrowser.defaultBrowser().registerObject(object);
    }

    public String description() {
        return object.toString();
    }

    // Invoked from takeValueForKey().
    // If the method returns value to adapt for takeValueForKey() of
    // object of the inspector.
    // Default implementation returns null.
    public Object inspectorWillTakeValueForKey(Object value, String key) {
        return null;
    }

    public void inspectorDidSetObject() {}
    public void inspectorDidInvokeAction() {}
    public void updateLoggers() {}
}
